@@ -31,7 +31,7 @@ static inline void outc(cmark_renderer *renderer, cmark_escaping escape,
         (c == '*' || c == '_' || c == '[' || c == ']' || c == '#' || c == '<' ||
          c == '>' || c == '\\' || c == '`' || c == '!' ||
          (c == '&' && isalpha(nextc)) || (c == '!' && nextc == '[') ||
-         (renderer->begin_line && (c == '-' || c == '+' || c == '=')) ||
+         (renderer->begin_content && (c == '-' || c == '+' || c == '=')) ||
          ((c == '.' || c == ')') &&
           isdigit(renderer->buffer->ptr[renderer->buffer->size - 1])))) ||
        (escape == URL && (c == '`' || c == '<' || c == '>' || isspace(c) ||
@@ -95,7 +95,7 @@ static int shortest_unused_backtick_sequence(const char *code) {
     used = used >> 1;
     i++;
   }
-  return i;
+  return (int)i;
 }
 
 static bool is_autolink(cmark_node *node) {
@@ -176,6 +176,7 @@ static int S_render_node(cmark_renderer *renderer, cmark_node *node,
   case CMARK_NODE_BLOCK_QUOTE:
     if (entering) {
       LIT("> ");
+      renderer->begin_content = true;
       cmark_strbuf_puts(renderer->prefix, "> ");
     } else {
       cmark_strbuf_truncate(renderer->prefix, renderer->prefix->size - 2);
@@ -214,9 +215,11 @@ static int S_render_node(cmark_renderer *renderer, cmark_node *node,
     if (entering) {
       if (cmark_node_get_list_type(node->parent) == CMARK_BULLET_LIST) {
         LIT("* ");
+        renderer->begin_content = true;
         cmark_strbuf_puts(renderer->prefix, "  ");
       } else {
         LIT(listmarker);
+        renderer->begin_content = true;
         for (i = marker_width; i--;) {
           cmark_strbuf_putc(renderer->prefix, ' ');
         }
@@ -228,12 +231,13 @@ static int S_render_node(cmark_renderer *renderer, cmark_node *node,
     }
     break;
 
-  case CMARK_NODE_HEADER:
+  case CMARK_NODE_HEADING:
     if (entering) {
-      for (int i = cmark_node_get_header_level(node); i > 0; i--) {
+      for (i = cmark_node_get_heading_level(node); i > 0; i--) {
         LIT("#");
       }
       LIT(" ");
+      renderer->begin_content = true;
       renderer->no_wrap = true;
     } else {
       renderer->no_wrap = false;
@@ -279,13 +283,20 @@ static int S_render_node(cmark_renderer *renderer, cmark_node *node,
     BLANKLINE();
     break;
 
-  case CMARK_NODE_HTML:
+  case CMARK_NODE_HTML_BLOCK:
     BLANKLINE();
     OUT(cmark_node_get_literal(node), false, LITERAL);
     BLANKLINE();
     break;
 
-  case CMARK_NODE_HRULE:
+  case CMARK_NODE_CUSTOM_BLOCK:
+    BLANKLINE();
+    OUT(entering ? cmark_node_get_on_enter(node) : cmark_node_get_on_exit(node),
+        false, LITERAL);
+    BLANKLINE();
+    break;
+
+  case CMARK_NODE_THEMATIC_BREAK:
     BLANKLINE();
     LIT("-----");
     BLANKLINE();
@@ -335,10 +346,15 @@ static int S_render_node(cmark_renderer *renderer, cmark_node *node,
     }
     break;
 
-  case CMARK_NODE_INLINE_HTML:
+  case CMARK_NODE_HTML_INLINE:
     OUT(cmark_node_get_literal(node), false, LITERAL);
     break;
 
+  case CMARK_NODE_CUSTOM_INLINE:
+    OUT(entering ? cmark_node_get_on_enter(node) : cmark_node_get_on_exit(node),
+        false, LITERAL);
+    break;
+
   case CMARK_NODE_STRONG:
     if (entering) {
       LIT("**");
@@ -368,9 +384,9 @@ static int S_render_node(cmark_renderer *renderer, cmark_node *node,
       if (entering) {
         LIT("<");
         if (strncmp(cmark_node_get_url(node), "mailto:", 7) == 0) {
-          LIT((char *)cmark_node_get_url(node) + 7);
+          LIT((const char *)cmark_node_get_url(node) + 7);
         } else {
-          LIT((char *)cmark_node_get_url(node));
+          LIT((const char *)cmark_node_get_url(node));
         }
         LIT(">");
         // return signal to skip contents of node...
